﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class BrawlManager : MonoBehaviour
{
  public delegate void ComputerMoveSelectedEvent(int index, MoveType move);
  public delegate void HealthEvent(int newHealth);

  public static event ComputerMoveSelectedEvent OnComputerMoveSelectedEvent;
  public static event HealthEvent OnPlayerHealthChanged;
  public static event HealthEvent OnComputerHealthChanged;

  const int N_OF_STRIKES_PER_COMBINATION = 6;
  const int STARTING_PLAYER_HEALTH = 5;
  const int STARTING_COMPUTER_HEALTH = 3;
  readonly YieldInstruction TIME_BETWEEN_STRIKES = new WaitForSeconds(0.75f);
  const float MOVE_DELAY_MIN = 0.05f;
  const float MOVE_DELAY_MAX = 0.1f;

  public FrankController Player;
  public FrankController Computer;

  MoveType[] playerMoves = new MoveType[N_OF_STRIKES_PER_COMBINATION];
  MoveType[] computerMoves = new MoveType[N_OF_STRIKES_PER_COMBINATION];

  void OnEnable()
  {
    UIManager.OnPrepareForMatch += OnPrepareForMatch;
    UIManager.OnStartFighting += OnStartFighting;
    UIManager.OnPlayerMoveSelected += OnPlayerMoveSelected;
  }

  void OnDisable()
  {
    UIManager.OnPrepareForMatch -= OnPrepareForMatch;
    UIManager.OnStartFighting -= OnStartFighting;
    UIManager.OnPlayerMoveSelected -= OnPlayerMoveSelected;
  }

  void Start()
  {
    Player.Health = STARTING_PLAYER_HEALTH;

    EmitPlayerHealthChanged();
  }

  void PrepareComputerOpenent()
  {
    Computer.Health = STARTING_COMPUTER_HEALTH;
    EmitComputerHealthChanged();

    for (int i = 0; i < computerMoves.Length; ++i) {
      MoveType move = Moves.GetRandom();
      computerMoves[i] = move;
      EmitComputerMoveSelectedEvent(i, move);
    }
  }

  IEnumerator Fight()
  {
    yield return null;
    for (int i = 0; i < N_OF_STRIKES_PER_COMBINATION; ++i) {
      yield return TIME_BETWEEN_STRIKES;

      MoveType playerMove = playerMoves[i];
      bool canPlayerStrike = Player.HasStrikeLimb(playerMove);
      bool canPlayerDefend = Player.HasDefenseLimb(playerMove);

      MoveType computerMove = computerMoves[i];
      bool canComputerStrike = Computer.HasStrikeLimb(computerMove);
      bool canComputerDefend = Computer.HasDefenseLimb(computerMove);

      // TODO: add random order of delaying
      if (canPlayerStrike | canPlayerDefend) {
        Player.Strike(playerMove, computerMove, canComputerDefend);
        Computer.ReactTo(computerMove, playerMove);
        EmitComputerHealthChanged();
      }

      yield return GetRandomDelay();

      if (canComputerStrike | canComputerDefend) {
        Computer.Strike(computerMove, playerMove, canPlayerDefend);
        Player.ReactTo(playerMove, computerMove);
        EmitPlayerHealthChanged();
      }
    }

    Debug.Log("End of Fight");
  }

  YieldInstruction GetRandomDelay()
  {
    return new WaitForSeconds(Random.Range(MOVE_DELAY_MIN, MOVE_DELAY_MAX));
  }

  void OnPrepareForMatch()
  {
    PrepareComputerOpenent();
  }

  void OnStartFighting()
  {
    StartCoroutine(Fight());
  }

  void OnPlayerMoveSelected(int index, MoveType move)
  {
    playerMoves[index] = move;
  }

  void EmitComputerMoveSelectedEvent(int index, MoveType move)
  {
    if (OnComputerMoveSelectedEvent != null) {
      OnComputerMoveSelectedEvent(index, move);
    }
  }

  void EmitPlayerHealthChanged()
  {
    if (OnPlayerHealthChanged != null) {
      OnPlayerHealthChanged(Player.Health);
    }
  }

  void EmitComputerHealthChanged()
  {
    if (OnComputerHealthChanged != null) {
      OnComputerHealthChanged(Computer.Health);
    }
  }
}
