﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class UIManager : MonoBehaviour
{
  public delegate void UIMoveSelectedEvent(int index, MoveType move);
  public delegate void UIEvent();

  public static event UIEvent OnPrepareForMatch;
  public static event UIEvent OnStartFighting;
  public static event UIMoveSelectedEvent OnPlayerMoveSelected;

  public CircleCollider2D[] buttons;
  public SpriteRenderer[] selected;
  public SpriteRenderer[] playerCombo;
  public SpriteRenderer[] computerCombo;
  public BoxCollider2D startButton;
  private GameObject startButtonObj;

  public GameObject buttonsWrapper;
  public GameObject panel;
  public GameObject combosWrapper;

  public UnityEngine.UI.Text playerHealth;
  public UnityEngine.UI.Text computerHealth;

  int uiLayerMask;
  int currentSelected;

  MoveType[] movesMap;

  void OnEnable()
  {
    BrawlManager.OnComputerMoveSelectedEvent += OnComputerMoveSelectedEvent;
    BrawlManager.OnPlayerHealthChanged += OnPlayerHealthChanged;
    BrawlManager.OnComputerHealthChanged += OnComputerHealthChanged;
  }

  void OnDisable()
  {
    BrawlManager.OnComputerMoveSelectedEvent -= OnComputerMoveSelectedEvent;
    BrawlManager.OnPlayerHealthChanged -= OnPlayerHealthChanged;
    BrawlManager.OnComputerHealthChanged -= OnComputerHealthChanged;
  }

  void Start()
  {
    uiLayerMask = LayerMask.GetMask("UI");
    startButtonObj = startButton.gameObject;

    movesMap = new MoveType[] {
      MoveType.DEFEND_BALLS,
      MoveType.DEFEND_HEAD,
      MoveType.STRIKE_FORE_LEG,
      MoveType.STRIKE_BACK_LEG,
      MoveType.STRIKE_FORE_ARM,
      MoveType.STRIKE_BACK_ARM,
    };

    PreprareForMatch();
  }

  void Update()
  {
    if (IsEnoughSelected()) {
      ProcessStartButtonClick();
    } else {
      ProcessComboSelection();
    }
  }

  void PreprareForMatch()
  {
    buttonsWrapper.SetActive(true);
    panel.SetActive(true);
    combosWrapper.SetActive(false);
    startButtonObj.SetActive(false);

    currentSelected = 0;

    StartCoroutine(EmitPrepareForMatch());
  }

  void ProcessComboSelection()
  {
    if (Input.GetMouseButtonUp(0)) {
      Vector2 mousePosition = Camera.main.ScreenToWorldPoint(Input.mousePosition);
      Collider2D coll = Physics2D.OverlapPoint(mousePosition, uiLayerMask);

      if (coll != null) {
        for (int i = 0; i < buttons.Length; ++i) {
          if (coll == buttons[i]) {
            SpriteRenderer buttonRenderer = coll.GetComponent<SpriteRenderer>();
            selected[currentSelected].sprite = buttonRenderer.sprite;
            selected[currentSelected].flipX = buttonRenderer.flipX;
            playerCombo[currentSelected].sprite = buttonRenderer.sprite;
            playerCombo[currentSelected].flipX = buttonRenderer.flipX;

            EmitPlayerMoveSelected(currentSelected, movesMap[i]);

            currentSelected++;

            if (IsEnoughSelected()) {
              OnEnoughSelected();
            }
            break;
          }
        }
      }
    }
  }

  void ProcessStartButtonClick()
  {
    if (Input.GetMouseButtonUp(0)) {
      Vector2 mousePosition = Camera.main.ScreenToWorldPoint(Input.mousePosition);
      Collider2D coll = Physics2D.OverlapPoint(mousePosition, uiLayerMask);

      if (coll != null && coll == startButton) {
        OnStartClicked();
      }
    }
  }

  bool IsEnoughSelected()
  {
    return currentSelected >= selected.Length;
  }

  void OnEnoughSelected()
  {
    buttonsWrapper.SetActive(false);
    startButtonObj.SetActive(true);

    combosWrapper.SetActive(true);
  }

  void OnStartClicked()
  {
    panel.SetActive(false);
    EmitStartFighting();
  }

  IEnumerator EmitPrepareForMatch()
  {
    yield return new WaitForSeconds(0.1f); // TODO: to constant
    if (OnPrepareForMatch != null) {
      OnPrepareForMatch();
    }
  }

  void OnComputerMoveSelectedEvent(int index, MoveType move)
  {
    int moveIndex = 0;
    for (int i = 0; i < movesMap.Length; ++i) {
      if (movesMap[i] == move) {
        moveIndex = i;
        break;
      }
    }
    SpriteRenderer buttonRenderer = buttons[moveIndex].GetComponent<SpriteRenderer>();
    computerCombo[index].sprite = buttonRenderer.sprite;
    computerCombo[index].flipX = buttonRenderer.flipX;
  }

  void EmitStartFighting()
  {
    if (OnStartFighting != null) {
      OnStartFighting();
    }
  }

  void EmitPlayerMoveSelected(int index, MoveType move)
  {
    if (OnPlayerMoveSelected != null) {
      OnPlayerMoveSelected(index, move);
    }
  }

  void OnPlayerHealthChanged(int newHealth)
  {
    UpdateHealthText(playerHealth, newHealth);
  }

  void OnComputerHealthChanged(int newHealth)
  {
    UpdateHealthText(computerHealth, newHealth);
  }

  void UpdateHealthText(UnityEngine.UI.Text text, int healthString)
  {
    text.text = string.Format("Health: {0}", healthString);
  }
}
